<?php

use Tygh\Http;
use Tygh\Registry;

if (!defined('BOOTSTRAP')) { die('Access denied'); }

if ($_SERVER['REQUEST_METHOD'] == 'POST') {

    return;

}

if ($mode == 'transfer_data') {
    define("AUTH_URL",  "https://api.waymore.io/api/authenticate");
    writeLog('auth', 'auth', 200, "event triggered");
    $routee_uuid = db_get_field('SELECT description FROM ?:common_descriptions WHERE object_holder = ?s AND lang_code = ?s', 'routee_uuid', 'en');

    try {

        if (!empty($routee_uuid)) {

            fn_set_notification('N', __('notice'), __('routee.already_authorized'));
            exit;

        }
        $auth_data = array(
            'username' => Registry::get('addons.routee.username'),
            'password' => Registry::get('addons.routee.password'),
            'source' => 'CSCart',
            'type' => 'e-commerce',
            'version' => PRODUCT_VERSION,
            'plugin_version' => '1.5.2',
            'domain' => Registry::get('config.current_location'),
        );
		
        writeLog('event', 'Search', 200, "event ready for api call");
        $return = json_decode(Http::post(AUTH_URL, json_encode($auth_data)), true);
		
        writeLog('event', 'Search', 200, "event finished");

        if (isset($return['uuid']) && !empty($return['uuid'])) {

            $_data = array(
                'object_id' => '1',
                'lang_code' => 'en',
                'description' => $return['uuid'],
                'object_holder' => 'routee_uuid'
            );
            db_query('REPLACE INTO ?:common_descriptions ?e', $_data);

            fn_set_notification('N', __('notice'), __('routee.authorization_successful'));

        } else {

            fn_set_notification('E', __('error'), __('routee.authorization_refused'));
            exit;

        }
    }catch (Exception $exception){
        writeLog('auth', 'auth', $exception->getCode(), $exception->getMessage());
    }

	$routee_uuid = db_get_field('SELECT description FROM ?:common_descriptions WHERE object_holder = ?s AND lang_code = ?s', 'routee_uuid', 'en');

	if (empty($routee_uuid)) {
		exit;
	}
	
	// transfer products

    try {
        writeLog('mass', 'products', 200, "event triggered");
        fn_set_progress('echo', 'Preparing products...', false);

        list($products, $search) = fn_get_products(array('extend' => array('description')));
        fn_gather_additional_products_data($products, array(
            'get_icon' => true,
            'get_detailed' => true,
            'get_additional' => true,
            'get_options' => true,
            'get_discounts' => false,
            'get_features' => false
        ));

        fn_set_progress('parts', $search['total_items']);

        if (!empty($products)) {

            $p = 0;
            $i = 0;

            $mass_data = array(
                'uuid' => $routee_uuid,
                'data' => array(
                    array(
                        'name' => 'products',
                        'description' => 'shop products'
                    )
                )
            );

            foreach ($products as $v) {

                $mass_data['data'][0]['object'][$i] = array(
                    'product_id' => $v['product_id'],
                    'name' => $v['product'],
                    'description' => $v['full_description'],
                    'short_description' => $v['short_description'],
                    'stock_quantity' => $v['amount'],
                    'price' => $v['price'],
                    'image_link' => (!empty($v['main_pair'])) ? $v['main_pair']['detailed']['image_path'] : '',
                    'product_link' => fn_url('products.view?product_id='.$v['product_id'], 'C'),
                );
                if (!empty($v['category_ids'])) {
                    foreach ($v['category_ids'] as $vv) {
                        $mass_data['data'][0]['object'][$i]['categories'][] = fn_get_category_name($vv);
                    }
                }
                if (!empty($v['product_options'])) {
                    foreach ($v['product_options'] as $opt) {
                        if (!empty($opt['variants'])) {
                            foreach ($opt['variants'] as $var) {
                                $mass_data['data'][0]['object'][$i]['product_options'][] = array(
                                    'option_id' => $opt['option_id'],
                                    'option_name' => $opt['option_name'],
                                    'varian_name' => $var['variant_name'],
                                    'price_modifier' => $var['modifier'],
                                );
                            }
                        }
                    }
                }

                $i++;
                $p++;
                fn_set_progress('echo','Transferring products... '.$p.' of '.$search['total_items']);

                if ($i == 100 || $p == $search['total_items']) {

                    writeLog('mass', 'products', 200, "event ready for api call");
                    $return = Http::post('https://idata.routee.net/api/data', json_encode($mass_data));
                    writeLog('mass', 'products', 200, "event finished");

                    $i = 0;
                    $mass_data['data'][0]['object'] = array();

                }

            }

        }
    }catch (Exception $exception){
        writeLog('mass', 'products', $exception->getCode(), $exception->getMessage());
    }
	
	// transfer customers

    try {
        writeLog('mass', 'customers', 200, "event triggered");
        fn_set_progress('echo', 'Preparing customers...', false);

        list($users, $search) = fn_get_users(array(), $_SESSION['auth']);

        if (!empty($users)) {

            fn_set_progress('parts', sizeof($users));

            $u = 0;
            $i = 0;
            $total_users = sizeof($users);

            $mass_data = array(
                'uuid' => $routee_uuid,
                'data' => array(
                    array(
                        'name' => 'customers',
                        'description' => 'shop customers'
                    )
                )
            );

            foreach ($users as $v) {

                $user_data = fn_get_user_info($v['user_id']);

                $mass_data['data'][0]['object'][$i] = array(
                    'customer' => array(
                        'id' => $user_data['user_id'],
                        'lastname' => $user_data['firstname'],
                        'firstname' => $user_data['lastname'],
                        'email' => $user_data['email'],
                        'phone' => $user_data['phone'],
                    ),
                    'addresses' => array(
                        array(
                            'country' => $user_data['b_country_descr'],
                            'city' => $user_data['b_city'],
                            'address1' => $user_data['b_address'],
                            'postcode' => $user_data['b_zipcode'],
                            'company' => $user_data['company'],
                        )
                    )
                );

                $i++;
                $u++;
                fn_set_progress('echo','Transferring users... '.$u.' of '.$total_users);

                if ($i == 100 || $u == $total_users) {

                    writeLog('mass', 'customers', 200, "event ready for api call");
                    $return = Http::post('https://idata.routee.net/api/data', json_encode($mass_data));
                    writeLog('mass', 'customers', 200, "event finished");

                    $i = 0;
                    $mass_data['data'][0]['object'] = array();

                }

            }

        }
    }catch (Exception $exception){
        writeLog('mass', 'customers', $exception->getCode(), $exception->getMessage());
    }
	
	// transfer orders

    try {
        writeLog('mass', 'orders', 200, "event triggered");
        fn_set_progress('echo', 'Preparing orders...', false);

        list($orders, $search) = fn_get_orders(array());

        if (!empty($orders)) {

            $o = 0;
            $i = 0;
            $total_orders = sizeof($orders);

            fn_set_progress('parts', $total_orders);

            $mass_data = array(
                'uuid' => $routee_uuid,
                'data' => array(
                    array(
                        'name' => 'orders',
                        'description' => 'shop orders'
                    )
                )
            );

            foreach ($orders as $v) {

                $order_info = fn_get_order_info($v['order_id'], false, true, false, false);

                $mass_data['data'][0]['object'][$i] = array(
                    'order_details' => array(
                        'order_id' => $order_info['user_id'],
                        'order_status' => db_get_field('SELECT ?:status_descriptions.description FROM ?:status_descriptions LEFT JOIN ?:statuses ON ?:status_descriptions.status_id = ?:statuses.status_id WHERE ?:statuses.status = ?s AND ?:statuses.type = ?s AND ?:status_descriptions.lang_code = ?s', ($order_info['status'] == 'N') ? 'O' : $order_info['status'], 'O', CART_LANGUAGE),
                        'shipping_method' => db_get_field('SELECT ?:shipping_descriptions.shipping FROM ?:shipping_descriptions LEFT JOIN ?:shippings ON ?:shippings.shipping_id = ?:shippings.shipping_id WHERE ?:shippings.shipping_id = ?i AND ?:shipping_descriptions.lang_code = ?s', $order_info['shipping_ids'], CART_LANGUAGE),
                        'payment_method' => isset($order_info['payment_method']['payment']) ? $order_info['payment_method']['payment'] : '',
                        'total_price' => $order_info['total'],
                    )
                );
                if (!empty($order_info['products'])) {
                    foreach ($order_info['products'] as $v) {
                        $mass_data['data'][0]['object'][$i]['order_products'][] = array(
                            'product_id' => $v['product_id'],
                            'quantity' => $v['amount'],
                        );
                    }
                }

                $i++;
                $o++;
                fn_set_progress('echo','Transferring orders... '.$o.' of '.$total_orders);

                if ($i == 100 || $o == $total_orders) {

                    writeLog('mass', 'orders', 200, "event ready for api call");
                    $return = Http::post('https://idata.routee.net/api/data', json_encode($mass_data));
                    writeLog('mass', 'orders', 200, "event finished");

                    $i = 0;
                    $mass_data['data'][0]['object'] = array();

                }

            }

        }
    }catch (Exception $exception){
        writeLog('mass', 'orders', $exception->getCode(), $exception->getMessage());
    }

	fn_set_notification('N', __('notice'), __('done'));
	exit;
	
}

function eventName($eventType) {
    $name = '';

    switch ($eventType) {
        case 1:
            $name = "auth";
            break;
        case 3:
            $name = "massdata";
            break;
        case 2:
            $name = "events";
            break;
    }

    return $name;
}

if ($mode == 'export_log') {
    $routee_uuid = db_get_field('SELECT description FROM ?:common_descriptions WHERE object_holder = ?s AND lang_code = ?s', 'routee_uuid', 'en');

    try {

//        if (!empty($routee_uuid)) {
//            fn_set_notification('N', __('notice'), "Exporting..");
//            exit;
//        }

        $resArr = [];
        $shop = Registry::get('config.current_location');
        $select_sql = db_get_array("SELECT * FROM `routee_shop_logs` WHERE is_exported=0");
            $postArr = $ids = [];
            $count = count($select_sql);
            if ( $count > 0 ) {
                foreach ($select_sql as $key => $log) {
                    $ids[] = $log->id;
                    $postArr[] = array(
                        'siteUrl' => $shop,
                        'uuid' => $routee_uuid,
                        'event_name' => $this->eventName($log['event_type']),
                        'log_type' => $log['log_type'] == 1 ? 'success' : 'failure',
                        'log_data' => $log['log_data'],
                        'created_at' => $log['created_at'],
                        'platform' => "CsCart"
                    );
                }
                $result = Http::post('https://autoforms.routee.net/u001/log.php?multi=1', $postArr);
                $qry = "UPDATE routee_event_logs SET is_exported=1 WHERE is_exported=0 AND id IN (" . implode(',', $ids) . ")";
                db_query($qry);
            }
        exit;

    }catch (Exception $exception){

    }

    fn_set_notification('N', __('notice'), __('done'));
    exit;

}



if ($mode == 'export_log_csv') {
    $routee_uuid = db_get_field('SELECT description FROM ?:common_descriptions WHERE object_holder = ?s AND lang_code = ?s', 'routee_uuid', 'en');
    try {

//        if (!empty($routee_uuid)) {
//            fn_set_notification('N', __('notice'), "Exporting..");
//            exit;
//        }

        $resArr = [];
        $shop = Registry::get('config.current_location');
        $select_sql = db_get_array("SELECT * FROM `routee_shop_logs` WHERE is_exported=0");
        if (count($select_sql) > 0){
            array_to_csv_download($select_sql,
                "routee_logs.csv"
            );
        }

    }catch (Exception $exception){

    }

    fn_set_notification('N', __('notice'), __('done'));
    exit;

}

function array_to_csv_download($array, $filename = "routee_logs.csv", $delimiter=";") {
    $f = fopen('php://memory', 'w');
    foreach ($array as $line) {
        fputcsv($f, $line, $delimiter);
    }
    fseek($f, 0);
    header('Content-Type: text/csv');
    header('Content-Disposition: attachment; filename="'.$filename.'";');
    fpassthru($f);
}


/*function writeLog( $mode, $event, $code, $msg ) {
    $shop = Registry::get('config.current_location');
    $data = array(
        'mode' => $mode,
        'event' => $event,
        'code' => $code,
        'exception_message' => $msg
    );
    $logType = $code !== 200 ? 'failure' : "success";
    saveLogs($event, $data, $shop, $logType);
    sendLog($shop,$event, $logType,$data);
}

function saveLogs($eventType, $data, $shop, $logType){
    $isExported = 0;
    $data = json_encode($data);
    $time = gmdate('d-m-Y H:i:s');
    db_query("INSERT INTO routee_event_logs (store_url, log_type,event_type,log_data,is_exported,created_at) VALUES ('{$shop}','{$logType}','{$eventType}','{$data}','{$isExported}','{$time}')");
}

function sendLog($shop, $event, $logType, $data){
    if ($logType == 'failure'){
        $routee_uuid = db_get_field('SELECT description FROM ?:common_descriptions WHERE object_holder = ?s AND lang_code = ?s', 'routee_uuid', 'en');
        $postArr = array(
            'siteUrl' => $shop,
            'uuid' => $routee_uuid,
            'event_name' => $event,
            'log_type' => 'failure',
            'log_data' => $data,
            'created_at' => gmdate('d-m-Y H:i:s'),
            'platform' => "CSCart"
        );
        Http::post("https://autoforms.routee.net/u001/log.php",json_encode($postArr));
    }
}*/